@@ -174,6 +174,24 @@ ww_multi_scale.SpatRaster <- function(
   rlang::check_installed("terra")
   rlang::check_installed("exactextractr")
 
+  data <- prep_multi_scale_raster(data, truth, estimate)
+  metrics <- handle_metrics(metrics)
+  grid_list <- handle_grids(data, grids, autoexpand_grid, sf::st_crs(data), ...)
+
+  grid_list$grids <- lapply(
+    grid_list$grids,
+    spatraster_extract,
+    data,
+    aggregation_function,
+    progress
+  )
+
+  .notes <- raster_method_notes(grid_list)
+
+  raster_method_summary(grid_list, .notes, metrics, na_rm)
+}
+
+prep_multi_scale_raster <- function(data, truth, estimate) {
   data <- tryCatch(
     terra::subset(data, c(truth, estimate)),
     error = function(e) {
@@ -188,52 +206,30 @@ ww_multi_scale.SpatRaster <- function(
     ))
   }
   names(data) <- c("truth", "estimate")
+  data
+}
 
-  metrics <- handle_metrics(metrics)
-  grid_list <- handle_grids(data, grids, autoexpand_grid, ...)
-
-  grid_list$grids <- purrr::map(
-    grid_list$grids,
-    function(grid) {
-      grid <- sf::st_as_sf(grid)
-      sf::st_geometry(grid) <- "geometry"
-      if (rlang::is_function(aggregation_function) || aggregation_function == "count") {
-        grid <- cbind(
-          grid,
-          stats::setNames(
-            exactextractr::exact_extract(
-              data,
-              grid,
-              fun = aggregation_function,
-              progress = progress
-            ),
-            c(".truth", ".estimate")
-          )
-        )
-        grid[c(".truth", ".estimate", "geometry")]
-      } else {
-        grid_df <- exactextractr::exact_extract(
-          data,
-          grid,
-          fun = c(aggregation_function, "count"),
-          progress = progress
-        )
-        names(grid_df) <- c(".truth", ".estimate", ".truth_count", ".estimate_count")
-
-        cbind(grid, grid_df)[c(
-          ".truth",
-          ".truth_count",
-          ".estimate",
-          ".estimate_count",
-          "geometry"
-        )]
-      }
-    }
+spatraster_extract <- function(grid, data, aggregation_function, progress) {
+  grid <- sf::st_as_sf(grid)
+  sf::st_geometry(grid) <- "geometry"
+  exactextract_names <- c(".truth", ".estimate")
+  if (!rlang::is_function(aggregation_function) && aggregation_function != "count") {
+    exactextract_names <- c(exactextract_names, ".truth_count", ".estimate_count")
+    aggregation_function <- c(aggregation_function, "count")
+  }
+  grid_df <- exactextractr::exact_extract(
+    data,
+    grid,
+    fun = aggregation_function,
+    progress = progress
   )
+  names(grid_df) <- exactextract_names
 
-  .notes <- raster_method_notes(grid_list)
+  if (length(exactextract_names) == 4L) {
+    exactextract_names <- exactextract_names[c(1, 3, 2, 4)]
+  }
 
-  raster_method_summary(grid_list, .notes, metrics, na_rm)
+  cbind(grid, grid_df)[c(exactextract_names, "geometry")]
 }
 
 ww_multi_scale_raster_args <- function(
@@ -263,9 +259,9 @@ ww_multi_scale_raster_args <- function(
   }
 
   metrics <- handle_metrics(metrics)
-  grid_list <- handle_grids(truth, grids, autoexpand_grid, ...)
+  grid_list <- handle_grids(truth, grids, autoexpand_grid, sf::st_crs(data), ...)
 
-  grid_list$grids <- purrr::map(
+  grid_list$grids <- lapply(
     grid_list$grids,
     function(grid) {
       grid <- sf::st_as_sf(grid)
@@ -318,7 +314,7 @@ ww_multi_scale_raster_args <- function(
 }
 
 raster_method_notes <- function(grid_list) {
-  purrr::map(
+  lapply(
     seq_along(grid_list$grids),
     function(idx) {
       tibble::tibble(
@@ -330,12 +326,7 @@ raster_method_notes <- function(grid_list) {
 }
 
 raster_method_summary <- function(grid_list, .notes, metrics, na_rm) {
-  purrr::pmap_dfr(
-    list(
-      grid = grid_list$grids,
-      grid_arg = grid_list$grid_arg_idx,
-      .notes = .notes
-    ),
+  out <- mapply(
     function(grid, grid_arg, .notes) {
       out <- metrics(grid, .truth, .estimate, na_rm = na_rm)
       out[attr(out, "sf_column")] <- NULL
@@ -343,8 +334,13 @@ raster_method_summary <- function(grid_list, .notes, metrics, na_rm) {
       out$.grid <- list(grid)
       out$.notes <- list(.notes)
       out
-    }
+    },
+    grid = grid_list$grids,
+    grid_arg = grid_list$grid_arg_idx,
+    .notes = .notes,
+    SIMPLIFY = FALSE
   )
+  do.call(dplyr::bind_rows, out)
 }
 
 #' @exportS3Method
@@ -365,23 +361,7 @@ ww_multi_scale.sf <- function(
     )
   }
 
-  if (any(names(data) %in% c(".truth", ".estimate", ".truth_count", ".estimate_count"))) {
-    rlang::abort(c(
-      "This function cannot work with data whose columns are named `.truth`, `.estimate`, `.truth_count`, or `estimate_count`.",
-      i = "Rename the relevant columns and try again."
-    ))
-  }
-
-  geom_type <- unique(sf::st_geometry_type(data))
-  if (!(length(geom_type) == 1 && geom_type == "POINT")) {
-    rlang::abort(
-      c(
-        "ww_multi_scale is currently only implemented for point geometries.",
-        i = "Consider casting your data to points."
-      )
-    )
-  }
-
+  check_multi_scale_data(data)
   metrics <- handle_metrics(metrics)
 
   truth_var <- tidyselect::eval_select(rlang::expr({{ truth }}), data)
@@ -395,29 +375,17 @@ ww_multi_scale.sf <- function(
     rlang::abort("`estimate` must be numeric.")
   }
 
-  grid_list <- handle_grids(data, grids, autoexpand_grid, ...)
+  data_crs <- sf::st_crs(data)
+  grid_list <- handle_grids(data, grids, autoexpand_grid, data_crs, ...)
 
   data$.grid_idx <- seq_len(nrow(data))
-  out <- purrr::map2_dfr(
-    grid_list$grids,
-    grid_list$grid_arg_idx,
+  out <- mapply(
     function(grid, grid_args_idx) {
       grid_args <- grid_list[["grid_args"]][grid_args_idx, ]
 
-      grid <- sf::st_as_sf(grid)
-
-      data_crs <- sf::st_crs(data)
-      grid_crs <- sf::st_crs(grid)
-      # If both have a CRS, reproject
-      if (!is.na(data_crs) && !is.na(grid_crs)) {
-        grid <- sf::st_transform(grid, data_crs)
-        # if only data has CRS, assume grid in same
-      } else if (!is.na(data_crs)) {
-        grid <- sf::st_set_crs(grid, data_crs)
-      }
-      # if neither has a CRS, ignore (so, implicitly assume grid is in same)
-
+      grid <- prep_grid(data, grid, data_crs)
       grid$grid_cell_idx <- seq_len(nrow(grid))
+
       grid_matches <- sf::st_join(
         grid,
         data[".grid_idx"],
@@ -425,45 +393,17 @@ ww_multi_scale.sf <- function(
       )
       grid_matches <- sf::st_drop_geometry(grid_matches)
 
-      missing <- setdiff(data[[".grid_idx"]], grid_matches[[".grid_idx"]])
-      note <- character(0)
-      if (length(missing) > 0) {
-        note <- "Some observations were not within any grid cell, and as such were not used in any assessments. Their row numbers are in the `missing_indices` column."
-        missing <- list(missing)
-      } else {
-        missing <- list()
-      }
+      notes_tibble <- make_notes_tibble(data, grid_matches)
 
-      notes_tibble <- tibble::tibble(
-        note = note,
-        missing_indices = missing
-      )
-
-      matched_data <- dplyr::left_join(
+      matched_data <- match_data(
         data,
         grid_matches,
-        by = dplyr::join_by(.grid_idx)
-      )
-      matched_data <- sf::st_drop_geometry(matched_data)
-      matched_data <- matched_data[!is.na(matched_data[["grid_cell_idx"]]), ]
-      matched_data <- dplyr::group_by(
         matched_data,
-        dplyr::across(dplyr::all_of(c(dplyr::group_vars(data), "grid_cell_idx")))
+        truth_var,
+        estimate_var,
+        aggregation_function
       )
 
-      matched_data <- dplyr::summarise(
-        matched_data,
-        .truth = rlang::exec(.env[["aggregation_function"]], .data[[names(truth_var)]]),
-        .truth_count = sum(!is.na(.data[[names(truth_var)]])),
-        .estimate = rlang::exec(.env[["aggregation_function"]], .data[[names(estimate_var)]]),
-        .estimate_count = sum(!is.na(.data[[names(estimate_var)]])),
-        .groups = "drop"
-      )
-
-      if (dplyr::is_grouped_df(data)) {
-        matched_data <- dplyr::group_by(matched_data, !!!dplyr::groups(data))
-      }
-
       out <- metrics(matched_data, .truth, .estimate, na_rm = na_rm)
       out["grid_cell_idx"] <- NULL
       out[attr(out, "sf_column")] <- NULL
@@ -477,11 +417,14 @@ ww_multi_scale.sf <- function(
       out$.grid <- list(.grid)
       out$.notes <- list(notes_tibble)
       out
-
-    }
+    },
+    grid = grid_list$grids,
+    grid_args_idx = grid_list$grid_arg_idx,
+    SIMPLIFY = FALSE
   )
+  out <- dplyr::bind_rows(out)
 
-  if (any(purrr::map_lgl(out[[".notes"]], function(x) nrow(x) > 0))) {
+  if (any(vapply(out[[".notes"]], function(x) nrow(x) > 0, logical(1)))) {
     rlang::warn(
       c(
         "Some observations were not within any grid cell, and as such were not used in any assessments.",
@@ -491,7 +434,6 @@ ww_multi_scale.sf <- function(
   }
 
   out
-
 }
 
 handle_metrics <- function(metrics) {
@@ -502,9 +444,19 @@ handle_metrics <- function(metrics) {
   metrics
 }
 
-handle_grids <- function(data, grids, autoexpand_grid, ...) {
+handle_grids <- function(data, grids, autoexpand_grid, data_crs, ...) {
   if (is.null(grids)) {
     grid_args <- rlang::list2(...)
+    if ("crs" %in% names(grid_args)) {
+      rlang::warn(
+        c(
+          "The `crs` argument (passed via `...`) will be ignored.",
+          i = "Grids will be created using the same crs as `data`."
+        ),
+        call = rlang::caller_env()
+      )
+      grid_args["crs"] <- NULL
+    }
     grid_arg_idx <- max(vapply(grid_args, length, integer(1)))
     grid_args <- stats::setNames(
       lapply(
@@ -538,17 +490,17 @@ handle_grids <- function(data, grids, autoexpand_grid, ...) {
         names(arg) <- names(grid_args)
         do.call(
           sf::st_make_grid,
-          c(x = list(grid_box), arg)
+          c(x = list(grid_box), crs = list(data_crs), arg)
         )
       }
     )
   } else {
+    rlang::check_dots_empty(call = rlang::caller_env())
     grid_args <- tibble::tibble()
     grid_arg_idx <- 0
-  }
-
-  if (!is.na(sf::st_crs(data))) {
-    grids <- purrr::map(grids, sf::st_transform, sf::st_crs(data))
+    if (!is.na(data_crs)) {
+      grids <- lapply(grids, sf::st_transform, sf::st_crs(data))
+    }
   }
 
   list(
@@ -558,6 +510,93 @@ handle_grids <- function(data, grids, autoexpand_grid, ...) {
   )
 }
 
+match_data <- function(data,
+                       grid_matches,
+                       matched_data,
+                       truth_var,
+                       estimate_var,
+                       aggregation_function) {
+  matched_data <- dplyr::left_join(
+    data,
+    grid_matches,
+    by = dplyr::join_by(.grid_idx)
+  )
+  matched_data <- sf::st_drop_geometry(matched_data)
+  matched_data <- matched_data[!is.na(matched_data[["grid_cell_idx"]]), ]
+  matched_data <- dplyr::group_by(
+    matched_data,
+    dplyr::across(dplyr::all_of(c(dplyr::group_vars(data), "grid_cell_idx")))
+  )
+
+  matched_data <- dplyr::summarise(
+    matched_data,
+    .truth = rlang::exec(.env[["aggregation_function"]], .data[[names(truth_var)]]),
+    .truth_count = sum(!is.na(.data[[names(truth_var)]])),
+    .estimate = rlang::exec(.env[["aggregation_function"]], .data[[names(estimate_var)]]),
+    .estimate_count = sum(!is.na(.data[[names(estimate_var)]])),
+    .groups = "drop"
+  )
+
+  if (dplyr::is_grouped_df(data)) {
+    matched_data <- dplyr::group_by(matched_data, !!!dplyr::groups(data))
+  }
+  matched_data
+}
+
+prep_grid <- function(data, grid, data_crs) {
+  grid <- sf::st_as_sf(grid)
+  grid_crs <- sf::st_crs(grid)
+  # If both have a CRS, reproject
+  if (!is.na(data_crs) && !is.na(grid_crs) && (grid_crs != data_crs)) {
+    grid <- sf::st_transform(grid, data_crs)
+    # if only data has CRS, assume grid in same
+  } else if (!is.na(data_crs)) {
+    grid <- sf::st_set_crs(grid, data_crs)
+  }
+  # if neither has a CRS, ignore (so, implicitly assume grid is in same)
+  grid
+}
+
+make_notes_tibble <- function(data, grid_matches) {
+  missing <- setdiff(data[[".grid_idx"]], grid_matches[[".grid_idx"]])
+  note <- character(0)
+  if (length(missing) > 0) {
+    note <- "Some observations were not within any grid cell, and as such were not used in any assessments. Their row numbers are in the `missing_indices` column."
+    missing <- list(missing)
+  } else {
+    missing <- list()
+  }
+
+  tibble::tibble(
+    note = note,
+    missing_indices = missing
+  )
+}
+
+check_multi_scale_data <- function(data) {
+  if (any(names(data) %in% c(".truth", ".estimate", ".truth_count", ".estimate_count"))) {
+    rlang::abort(
+      c(
+        "This function cannot work with data whose columns are named `.truth`, `.estimate`, `.truth_count`, or `estimate_count`.",
+        i = "Rename the relevant columns and try again."
+      ),
+      call = rlang::caller_env()
+    )
+  }
+
+  geom_type <- unique(sf::st_geometry_type(data))
+  if (!(length(geom_type) == 1 && geom_type == "POINT")) {
+    rlang::abort(
+      c(
+        "ww_multi_scale is currently only implemented for point geometries.",
+        i = "Consider casting your data to points."
+      ),
+      call = rlang::caller_env()
+    )
+  }
+
+}
+
 #' Expand geographic bounding boxes slightly
 #'
 #' Because we're drawing straight lines on spheres when working with geographic
